@@ -26,7 +26,7 @@ static short accuracy;
 static double ut12tt = 69.0;
 
 // Initialized quantities.
-static double lst, pos0[3], vel0[3], epos[3], evel[3], pobs[3], vobs[3];
+static double lst, pos0[3], vel0[3], epos[3], evel[3], pobs[3], vobs[3], spos[3], svel[3];
 
 static FILE *fp;
 static int idx = -1;
@@ -38,6 +38,16 @@ static double vlen(double *pos) {
   return sqrt(pos[0] * pos[0] + pos[1] * pos[1] + pos[2] * pos[2]);
 }
 
+static double vdist(const double *v1, const double *v2) {
+  double d2 = 0.0;
+  int i;
+  for(i = 3; --i >= 0;) {
+    const double d = v1[i] - v2[i];
+    d2 += d * d;
+  }
+  return sqrt(d2);
+}
+
 static void newline() {
   fprintf(fp, "\n%8.1f %-10s S%d O%d A%d: ", (tdb - J2000), source.name, source.type, obs.where, accuracy);
 }
@@ -62,7 +72,6 @@ static void openfile(const char *name) {
   else if(header) fprintf(fp, "%s", header);
 }
 
-
 static void printvector(double *v) {
   if(!v) fprintf(fp, "null ");
   fprintf(fp, "%12.6f %12.6f %12.6f ", v[0], v[1], v[2]);
@@ -72,8 +81,8 @@ static void printunitvector(double *v) {
   if(!v) fprintf(fp, "null ");
   else {
     double l = sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
-    if(accuracy == 0) fprintf(fp, "%12.9f %12.9f %12.9f ", v[0] / l, v[1] / l, v[2] / l);
-    else fprintf(fp, "%9.6f %9.6f %9.6f ", v[0] / l, v[1] / l, v[2] / l);
+    if(accuracy == 0) fprintf(fp, "%14.11f %14.11f %14.11f ", v[0] / l, v[1] / l, v[2] / l);
+    else fprintf(fp, "%11.8f %11.8f %11.8f ", v[0] / l, v[1] / l, v[2] / l);
   }
 }
 
@@ -202,15 +211,15 @@ static void test_mean_star() {
 
   openfile("mean_star");
   if(is_ok(mean_star(2433282.42345905, 10.0, -40.0, 1, &ra, &dec)))
-    fprintf(fp, "1 %12.6f %12.6f ", ra, dec);
+    fprintf(fp, "1 %12.9f %12.8f ", ra, dec);
 
   openfile("mean_star");
   if(is_ok(mean_star(2433282.42345905, 19.0, 30.0, 1, &ra, &dec)))
-    fprintf(fp, "2 %12.6f %12.6f ", ra, dec);
+    fprintf(fp, "2 %12.9f %12.8f ", ra, dec);
 
   openfile("mean_star");
   if(is_ok(mean_star(2433282.42345905, 2.7, 68.3, 1, &ra, &dec)))
-    fprintf(fp, "3 %12.6f %12.6f ", ra, dec);
+    fprintf(fp, "3 %12.9f %12.8f ", ra, dec);
 }
 
 
@@ -500,7 +509,7 @@ static void test_time_specific() {
 
 
 static int init() {
-  object earth;
+  object earth, sun;
   double tdb2[2] = { tdb };
 
   memset(pos0, 0, sizeof(pos0));
@@ -547,6 +556,20 @@ static int init() {
   printvector(evel);
   newline();
 
+  if(make_object(0, 10, "Sun", NULL, &sun) != 0) {
+    fprintf(stderr, "init: Failed make_object(Sun)\n");
+    return -1;
+  }
+  if(ephemeris(tdb2, &sun, 0, 1, spos, svel) != 0) {
+    fprintf(stderr, "init: Failed Earth ephemeris\n");
+    return -1;
+  }
+
+  fprintf(fp, "SUN ");
+  printvector(spos);
+  printvector(svel);
+  newline();
+
   if(sidereal_time(tdb, 0.0, ut12tt, 0, 1, accuracy, &lst) != 0) {
     fprintf(stderr, "init: Failed sidereal_time\n");
     return -1;
@@ -637,7 +660,10 @@ static void test_light_time() {
 }
 
 static void test_grav_def() {
-  double pos1[3];
+  double pos1[3], pos2[3], ps[3], vs[3];
+  double d, jd2[2] = { tdb };
+  object sun = { 0, 10, "Sun" };
+  int k;
 
   if(source.type != 2) return;
 
@@ -645,6 +671,16 @@ static void test_grav_def() {
   if(is_ok(grav_def(tdb, obs.where, accuracy, pos0, pobs, pos1))) {
     printunitvector(pos1);
   }
+
+  ephemeris(jd2, &sun, 0, accuracy, ps, vs);
+  d = vlen(pos0);
+
+  // Now test a position near the Sun in the direction of the source
+  for(k = 3; --k >= 0;) pos1[k] = ps[k] + 0.01 * pos0[k] / d;
+
+  if(is_ok(grav_def(tdb, obs.where, accuracy, pos1, pobs, pos2))) {
+    printunitvector(pos2);
+  }
 }
 
 static void test_aberration() {
@@ -909,8 +945,19 @@ static void test_rad_vel() {
   double rv = 0.0;
 
   openfile("rad_vel");
+
   rad_vel(&source, pos0, vel0, vobs, 0.0, 0.0, 0.0, &rv);
   fprintf(fp, "%12.6f ", rv);
+
+  rad_vel(&source, pos0, vel0, vobs, vdist(pobs, epos), vdist(pobs, spos), vdist(pos0, spos), &rv);
+  fprintf(fp, "%12.6f ", rv);
+}
+
+static void test_limb_angle() {
+  double limb = 0.0, nadir = 0.0;
+  openfile("limb_angle");
+  limb_angle(pos0, pobs, &limb, &nadir);
+  fprintf(fp, "%12.6f %12.6e ", limb, nadir);
 }
 
 static int test_source() {
@@ -928,6 +975,7 @@ static int test_source() {
   test_place();
   test_aberration();
   test_rad_vel();
+  test_limb_angle();
 
   if(obs.where == 0) {
     test_astro_place();
